/*

*/

import renderer.scene.*;
import renderer.scene.util.ModelShading;
import renderer.models_L.SierpinskiTriangle;
import renderer.pipeline.Pipeline;
import renderer.framebuffer.FrameBuffer;

import java.awt.Color;

/**

*/
public class SierpinskiMovie
{
   public static void main(String[] args)
   {
      final Scene scene = new Scene("SierpinskiMovie",
                                    Camera.projOrtho());

      // Create the Model object.
      final Model sierpinskiTriangle = new SierpinskiTriangle(8);
      ModelShading.setColor(sierpinskiTriangle.getNestedModel(0), Color.blue);
      ModelShading.setColor(sierpinskiTriangle.getNestedModel(1), Color.red);
      ModelShading.setColor(sierpinskiTriangle.getNestedModel(2), Color.magenta);

      // Create a Position object that will hold the Sierpinski triangle.
      final Position top_p = new Position(sierpinskiTriangle,
                                          "top",
                                          Matrix.rotateZ(90));
      scene.addPosition(top_p);


      // Create a framebuffer to render our scene into.
      final int width  = 1024;
      final int height = 1024;
      final FrameBuffer fb = new FrameBuffer(width, height, Color.black);

      for (int k = 0; k < 720; ++k)
      {
         fb.clearFB();
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_SierpinskiMovie_Frame%03d.ppm", k));
       //fb.dumpFB2File(String.format("PNG_SierpinskiMovie_Frame%03d.png", k), "png");

         updateNestedMatrices(sierpinskiTriangle, true);
      }
   }


   private static void updateNestedMatrices(final Model model, final boolean twoSubModel)
   {
      if (! model.nestedModels.isEmpty())
      {
         if (twoSubModel)
         {
            final Matrix mat1 = model.getNestedModel(1).matrix
                                                 .times(Matrix.rotateZ( 0.5));
            model.setNestedModel(1, model.getNestedModel(1).transform(mat1));

            final Matrix mat2 = model.getNestedModel(2).matrix
                                                 .times(Matrix.rotateZ(-0.5));
            model.setNestedModel(2, model.getNestedModel(2).transform(mat2));
         } else {
            final Matrix mat0 = model.getNestedModel(0).matrix
                                                 .times(Matrix.rotateZ( 0.5));
            model.setNestedModel(0, model.getNestedModel(0).transform(mat0));
         }
         for (Model m : model.nestedModels)
         {
            updateNestedMatrices(m, twoSubModel); // recursion
         }
      }
   }
}
